#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)

import sys
import logging
import os
import subprocess
import datetime

from utils.utils_files import *

class standard_compliance_test:

    def __init__(self,path,type=None):
        self.type = type
        self.path = path
        self.files = dir_or_file(path)
#        print("Files or directory checked:")
        self.gridded_files = ''
        self.other_files = ''
        self.formats1 = ''
        self.formats2 = ''

    def run_cfchecker(self,item):
        """RUN CFCHECKER FOR ONE FILE
        """
        cmd0 = "cfchecks --version auto %s" %item
        p0 = subprocess.Popen(
            cmd0, shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
            )
        out0, _ = p0.communicate()

        return out0

    def netcdf_test(self,args):
        """Check the CF COMPLIANCE FOR THE INPUT FILES
        """
        from subprocess import Popen
        for ncfile in args:

            string = '\nINFO: CF compliance test for file:\n'
            string += str(args) + '\n'
            out0 = self.run_cfchecker(ncfile)
            string = str(out0).replace('\'',"").replace("\"","").split('\\n')

            if any('ERROR:' in e for e in string):
                err = [item.split('ERROR:')[-1].strip() for item in string if item.startswith('ERROR:')]
                errstring = ncfile.split('/')[-1] + ' has ' + str(len(err))
                errstring += ' error messages' + '\n'
                errstring += '\n'.join(err)
                string += errstring
                logging.error(string)
            else:
                string += 'INFO: NO ERRORS FOUND when checking against CF'
                print('INFO: NO ERRORS FOUND when checking against CF')
                logging.info(string)
                pass

            # We are only interested on errors but could be interesting for 
            # further versions of the dqc_checker to analyze the warnings 
            # and info messages
            # wrn = [item.split('WARN:')[-1].strip() for item in string if item.startswith('WARN:')]
            # inf = [item.split('INFO:')[-1].strip() for item in string if item.startswith('INFO:')]

    def grib_test(self,args):
        """Check the GRIB BASELINE FOR THE INPUT FILES
        """
        import utils.grib_checker.grib_cck as gcck
        for gribfile in args:
            gcck.grib_cck_run(gribfile)

    def initiate_logging(self):
        # Initiate the logging
#        time0 = self.timefortitle
        path_logs = os.path.join(
            '/data',
            'DQC_LOGS'
        )

        if not os.path.exists(path_logs):
            os.makedirs(path_logs)

        default_log_name = '_'.join(
            [
                'LOG_TEST01_STANDARD_CHECK',
 #               (self.type).upper(),
 #               time0,
                '.log'
            ]
        )

        default_path = os.path.join(
            path_logs,
            default_log_name
        )
        
        logging.basicConfig(
            filename=default_path,
            format='%(levelname)s:%(asctime)s:%(message)s', 
            level=logging.INFO
        )

        stringA = '\n' + '#'*80 + '\n' + '#'*80
        stringA += '\n RUNNING STANDARD COMPLIANCE TEST FOR FOLDER:'
        stringA += '\n ' + self.path
        stringA += '\n AGAINST ' + (self.type).upper() + ' CONVENTION '
#        stringA += '\n EXECUTION TIME: ' + self.timeforinfo
        stringA += '\n' + '#'*80

        logging.info(stringA)

    def main(self):
        if self.type != None:

            if self.type == 'CF':
                ncfiles = [item for item in self.files if item.endswith('.nc')]
                if ncfiles != []:
                    self.initiate_logging()
                    self.netcdf_test(ncfiles)
                else:
                    raise Exception('No NetCDF files in the given input')
            else: 
                if self.type == 'grib':
                    gribfiles = [item for item in self.files if item.endswith('.grib')]
                    if gribfiles != []:
                        self.initiate_logging()
                        print("WARN: No CDM has been implemented yet for the variables to be checked - listing keys present in the files if grib2")                        
                        self.grib_test(gribfiles)
                    else:
                        raise Exception('No GRIB files in the given input')
                else:
                    raise Exception('Type not supported, type currently supported are grib and CF')
            

        else:
            print("WARN: No type selected, exiting")
            sys.exit()

if __name__ == "__main__":
    sys.exit()
