#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)

import sys
import logging
import os
import subprocess
from datetime import datetime

from cdo import Cdo

# This test will check the cds-form for the variable selected and check for
# gaps in leadtimes, years, etc
# It will not check files directly

from utils.utils_files import *

class temporal_completeness_test:

    def __init__(self, path, tstart, tend):
        self.path = path        
        self.files = dir_or_file(path)
        self.tstart = tstart
        self.tend = tend
        self.tstart_dt = datetime.strptime(tstart, '%Y-%m-%d')
        self.tend_dt = datetime.strptime(tend, '%Y-%m-%d')

    def initiate_logging(self):
        # Initiate the logging
#        time0 = self.timefortitle
        path_logs = os.path.join(
            '/data',
            'DQC_LOGS'
        )

        if not os.path.exists(path_logs):
            os.makedirs(path_logs)

        default_log_name = '_'.join(
            [
                'LOG_TEST03_temporal_COMPLETENESS_CHECK',
#                (self.type).upper(),
#                time0,
                '.log'
            ]
        )

        default_path = os.path.join(
            path_logs,
            default_log_name
        )
        
        logging.basicConfig(
            filename=default_path,
            format='%(levelname)s:%(asctime)s:%(message)s', 
            level=logging.INFO
        )

        stringA = '\n' + '#'*80 + '\n' + '#'*80
        stringA += '\n RUNNING temporal COMPLETENESS TEST FOR FOLDER:'
        stringA += '\n ' + self.path        
#        stringA += '\n EXECUTION TIME: ' + self.timeforinfo
        stringA += '\n' + '#'*80


    def check_timestamps(self):

    	# For all files
    	cdo = Cdo()
    	res = True
    	for f in self.files:
    		logging.info('Checking {}'.format(f))    		
    		try:
    			info = cdo.showtimestamp(input=f)
    		except:
    			logging.error('Bad file format (cdo cannot read file)')
    			print('ERROR: Bad file format ({})'.format(os.path.basename(f)))
    			res = False
    			break
	    	
	    	for i in info:
	    		for t in i.split():
	    			date = datetime.strptime(t, '%Y-%m-%dT%H:%M:%S')	    			
	    			if not(date >= self.tstart_dt and date <= self.tend_dt):	    			
	    				logging.warn('Found date out of range {}'.format(t, self.tstart, self.tend))
	    				print('WARN: Found date out of range {}'.format(t, self.tstart, self.tend))
	    				res = False

    	# Final message
    	if res:
    		logging.info('FILES ARE TEMPORALLY CONSISTENT')
    		print('INFO: FILES ARE TEMPORALLY CONSISTENT')
    	else:
    		logging.warn('TEMPORAL INCONSISTENCIES FOUND')
    		print('TEMPORAL INCONSISTENCIES FOUND')

    def main(self):

    	self.initiate_logging()
    	self.check_timestamps()
                
if __name__ == "__main__":
    sys.exit()
